#!/usr/bin/wish
#
######################################
#                                    #
#  PhyloGrapher by Alexander Kozik   #
#    version: December 01 2001       #
#                                    #
######################################

######################################
#        LIST OF PROCEDURES          #
######################################

##########################
#      GNU LICENSE       #
##########################
proc About_PhyloGrapher {} {

    set phylo_logo [image create photo -file ./Images/PhyloGrapherLogo.gif]
    set phylo_text "PhyloGrapher (Graph Visualization Tool)

Copyright (C) 2001 University of California at Davis
Department of Vegetable Crops, Michelmore lab., Alexander Kozik
    
This program is free software. You can redistribute it and/or modify it 
under the terms of the GNU General Public License ( http://www.gnu.org ) 
as published by the Free Software Foundation; either version 2 of the 
License, or (at your option) any later version.

In other words, you are free to modify, copy, or redistribute this source 
code and its documentation in any way you like, but you must distribute 
all derivative versions as free software under the same terms that we 
provided our code to you (i.e. the GNU General Public License). This 
precludes any use of the code in proprietary or commercial software 
unless your source code is made freely available.

This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along 
with this PhyloGrapher release, in the file LICENSE; if not, write to the 
Free Software Foundation, Inc., 675 Mass. Ave, Cambridge, MA 02139 USA."

    destroy .phylo_about
    toplevel .phylo_about 
    wm transient .phylo_about .
    wm title .phylo_about "About PhyloGrapher"
    wm protocol .phylo_about WM_DELETE_WINDOW "destroy .phylo_about"

    label .phylo_about.image -image $phylo_logo
    label .phylo_about.text -text $phylo_text -justify center
    button .phylo_about.ok -text "OK" -command "destroy .phylo_about"
    grid .phylo_about.image -padx 7 -pady 7
    grid .phylo_about.text -padx 2 -pady 2
    grid .phylo_about.ok -padx 2 -pady 2
}

proc HowTo_PhyloWebify {} {

    destroy .fly
    set t [Scrolled_Text .fly -width 60 -height 40]
    set in [open "README.txt" r]
    $t insert end [read $in]
    close $in
    wm title .fly "HowTo PhyloWebify"
    wm protocol .fly WM_DELETE_WINDOW "destroy .fly"
}

############ Scrolled Canvas ################
#            ---------------                #
#         Create Scrolled Canvas            #
#                                           #
#############################################
proc Scrolled_Canvas { phylo args } {
    toplevel $phylo

    wm title .phylo "PhyloGrapher Canvas"
    wm protocol .phylo WM_DELETE_WINDOW "destroy .phylo .geneid"

    eval {canvas $phylo.canvas -xscrollcommand [list $phylo.xscroll set] \
	  -yscrollcommand [list $phylo.yscroll set] \
	  -highlightthickness 0 -borderwidth 0} $args
    
    scrollbar $phylo.xscroll -orient horizontal \
	      -command [list $phylo.canvas xview]
    scrollbar $phylo.yscroll -orient vertical \
              -command [list $phylo.canvas yview]

    grid $phylo.canvas $phylo.yscroll -sticky news
    grid $phylo.xscroll -sticky ew
    grid rowconfigure $phylo 0 -weight 1
    grid columnconfigure $phylo 0 -weight 1
    return $phylo.canvas
}

proc Scrolled_Text { fly args } {
    toplevel $fly
    eval {text $fly.text -wrap none \
           -xscrollcommand [list $fly.xscroll set] \
           -yscrollcommand [list $fly.yscroll set]} $args
    scrollbar $fly.xscroll -orient horizontal \
              -command [list $fly.text xview]
    scrollbar $fly.yscroll -orient vertical \
              -command [list $fly.text yview]
    grid $fly.text $fly.yscroll -sticky news
    grid $fly.xscroll -sticky news
    grid rowconfigure $fly 0 -weight 1
    grid columnconfigure $fly 0 -weight 1
    button $fly.dismiss -text "Dismiss" -command "destroy .fly"
    grid $fly.dismiss
    
    return $fly.text
}

proc mkNode {x y current_gene_ID} {
    global nodeX nodeY edgeFirst edgeSecond
    global fake_identity_counter
    set fill_color blue4
    set outline_color green4
    if [regexp "Custom_Node" $current_gene_ID] {
	set fill_color white
	set outline_color red
    }
    set new [.phylo.canvas create oval [expr $x-5] [expr $y-5] \
	[expr $x+5] [expr $y+5] -fill $fill_color -outline $outline_color -tag $current_gene_ID]
    set nodeX($new) $x
    set nodeY($new) $y
    set edgeFirst($new) {}
    set edgeSecond($new) {}
    .phylo.canvas bind $current_gene_ID <Button-1> {
    set curX %x
    set curY %y
    }
    .phylo.canvas bind $current_gene_ID <B1-Motion> {
	moveNode [.phylo.canvas find withtag current] [expr %x-$curX] [expr %y-$curY]
	set curX %x
	set curY %y
    }
    bind .geneid.entry <Return> {
	Shake_the_Node [.phylo.canvas find withtag $current_gene_ID]
	showName $current_gene_ID .geneid.entry
    }
    .phylo.canvas bind $current_gene_ID <Button-2> {
	set x [.phylo.canvas canvasx %x]
	set x [expr int(round($x))]
	set y [.phylo.canvas canvasy %y]
	set y [expr int(round($y))]
	set curX [.phylo.canvas canvasx %x]
	set curX [expr int(round($curX))]
	set curY [.phylo.canvas canvasy %y]
	set curY [expr int(round($curY))]
	Print_Gene_ID $current_gene_ID $curX $curY $x $y
    }
    bind .phylo.canvas w {
	set curX %x
	set curY %y
	.phylo.canvas find withtag $current_gene_ID
	.phylo.canvas itemconfigure $current_gene_ID -fill white
    }
    bind .phylo.canvas b {
	set curX %x
	set curY %y
	.phylo.canvas find withtag $current_gene_ID
	.phylo.canvas itemconfigure $current_gene_ID -fill blue
    }
    bind .phylo.canvas c {
	set curX %x
	set curY %y
	.phylo.canvas find withtag $current_gene_ID
	.phylo.canvas itemconfigure $current_gene_ID -fill cyan
    }
    bind .phylo.canvas s {
	set curX %x
	set curY %y
	.phylo.canvas find withtag $current_gene_ID
	.phylo.canvas itemconfigure $current_gene_ID -fill blue4
    }
    bind .phylo.canvas g {
	set curX %x
	set curY %y
	.phylo.canvas find withtag $current_gene_ID
	.phylo.canvas itemconfigure $current_gene_ID -fill green
    }
    bind .phylo.canvas o {
	set curX %x
	set curY %y
	.phylo.canvas find withtag $current_gene_ID
	.phylo.canvas itemconfigure $current_gene_ID -fill orange
    }
    bind .phylo.canvas r {
	set curX %x
	set curY %y
	.phylo.canvas find withtag $current_gene_ID
	.phylo.canvas itemconfigure $current_gene_ID -fill red
    }
    bind .phylo.canvas v {
	set curX %x
	set curY %y
	.phylo.canvas find withtag $current_gene_ID
	.phylo.canvas itemconfigure $current_gene_ID -fill violet
    }
    bind .phylo.canvas p {
	set curX %x
	set curY %y
	.phylo.canvas find withtag $current_gene_ID
	.phylo.canvas itemconfigure $current_gene_ID -fill purple
    }
    bind .phylo.canvas 1 {
	set firstNode [.phylo.canvas find withtag $current_gene_ID]
	set current_gene_ID_A $current_gene_ID
	.phylo.canvas itemconfigure $current_gene_ID_A -fill purple
    }
    bind .phylo.canvas 2 {
	set curNode [.phylo.canvas find withtag $current_gene_ID]
	set current_gene_ID_B $current_gene_ID
	if {($firstNode != "") && ($curNode != "")} {
	    set fill_color green
	    set line_thickness 2
	    set fake_identity "fake_identity"
	    set fake_identity_value "0"
	    mkEdge $firstNode $curNode $fill_color $line_thickness $current_gene_ID_A $current_gene_ID_B $fake_identity\_$fake_identity_counter $fake_identity_value
	    .phylo.canvas itemconfigure $current_gene_ID_B -fill purple
	    incr fake_identity_counter
	}
    }
    .phylo.canvas bind $current_gene_ID <Enter> "showName $current_gene_ID .geneid.entry"
    .phylo.canvas bind $current_gene_ID <Leave> "clearName .geneid.entry .geneid.degree_entry"
    .phylo.canvas bind $current_gene_ID <Button-3> {
	set curX %x
	set curY %y
	Shake_the_Node [.phylo.canvas find withtag current]
    }
}

proc mkEdge {first second fill_color line_thickness current_gene_ID_A current_gene_ID_B edge_tag current_gene_similarity} {
    global nodeX nodeY edgeFirst edgeSecond edgeCounter
    set current_gene_similarity [expr int(round($current_gene_similarity*100))]
    set edge [.phylo.canvas create line $nodeX($first) $nodeY($first) \
	$nodeX($second) $nodeY($second) -fill $fill_color -width $line_thickness -tag $edge_tag\_$current_gene_similarity]

    .phylo.canvas lower $edge
    ###########################
    bind .phylo.canvas <Control-d> {
    set curX %x
    set curY %y
	.phylo.canvas find withtag current
	.phylo.canvas delete current
	clearName .geneid.entry .geneid.degree_entry
    }
    .phylo.canvas bind $edge_tag\_$current_gene_similarity <Enter> "HighLightEdge .phylo.canvas $edge_tag\_$current_gene_similarity $current_gene_similarity"
    ###########################
    .phylo.canvas bind $edge_tag\_$current_gene_similarity <Leave> "HighLightEdge_Back .phylo.canvas $edge_tag\_$current_gene_similarity $fill_color"
    ###########################
    .phylo.canvas bind $edge_tag\_$current_gene_similarity <Button-2> "Print_Edge_Weight $current_gene_similarity"
    ###########################
    lappend edgeFirst($first) $edge
    lappend edgeSecond($second) $edge
    update
    .phylo.canvas lower dummy_ps_background
    incr edgeCounter
}

proc moveNode_Slow {node xDist yDist} {
    global nodeX nodeY edgeFirst edgeSecond
    .phylo.canvas move $node $xDist $yDist
    incr nodeX($node) $xDist
    incr nodeY($node) $yDist
    set edge_counter 0
    foreach edge $edgeFirst($node) {
	.phylo.canvas coords $edge $nodeX($node) $nodeY($node) \
	[lindex [.phylo.canvas coords $edge] 2] \
	[lindex [.phylo.canvas coords $edge] 3]
	incr edge_counter
    }
    foreach edge $edgeSecond($node) {
	.phylo.canvas coords $edge [lindex [.phylo.canvas coords $edge] 0] \
	[lindex [.phylo.canvas coords $edge] 1] \
	$nodeX($node) $nodeY($node)
	incr edge_counter
    }
    showDegree $edge_counter .geneid.degree_entry
}

proc moveNode {node xDist yDist} {
    global nodeX nodeY edgeFirst edgeSecond
    .phylo.canvas move $node $xDist $yDist
    incr nodeX($node) $xDist
    incr nodeY($node) $yDist
    foreach edge $edgeFirst($node) {
	.phylo.canvas coords $edge $nodeX($node) $nodeY($node) \
	[lindex [.phylo.canvas coords $edge] 2] \
	[lindex [.phylo.canvas coords $edge] 3]
    }
    foreach edge $edgeSecond($node) {
	.phylo.canvas coords $edge [lindex [.phylo.canvas coords $edge] 0] \
	[lindex [.phylo.canvas coords $edge] 1] \
	$nodeX($node) $nodeY($node)
    }
}

proc Plot_Genes_on_Canvas { genes_range graph_radius } {

    global graph_data_file
    set tick 1
    set radians_per_degree [expr 3.141593/180]
    set graph_data_file_open [open $graph_data_file "r"]

    while {[gets $graph_data_file_open current_line_graph] >= 0} {
	set angle [expr $tick*(360.000001/$genes_range)]
	set current_gene_ID [lindex [split $current_line_graph "|"] 0]
	set pointX($tick) [expr int(round($graph_radius*sin($angle*$radians_per_degree))) \
		+ $graph_radius + 40]
	set pointY($tick) [expr int(round($graph_radius*cos($angle*$radians_per_degree))) \
		+ $graph_radius + 40]

	set dot_pointX($tick) [expr $pointX($tick) + 2]
	set dot_pointY($tick) [expr $pointY($tick) + 2]

	mkNode $dot_pointX($tick) $dot_pointY($tick) $current_gene_ID
	update
	incr tick
    }
    close $graph_data_file_open
}

proc Create_Graph_from_Data {} {

##### COMMENT ON WINDOWS CURSOR CONFIG #####
    .phylo.canvas config -cursor {watch white}

    global graph_data_file edgeCounter
    global level_of_similarity_1 level_of_similarity_2 level_of_similarity_3
    global level_of_similarity_4 level_of_similarity_5 level_of_similarity_6
    global color_of_similarity_1 color_of_similarity_2 color_of_similarity_3
    global color_of_similarity_4 color_of_similarity_5 color_of_similarity_6

    set graph_data_file_open [open $graph_data_file "r"]

    while {[gets $graph_data_file_open current_line_graph] >= 0 } {

	set current_gene_ID_A [lindex [split $current_line_graph "|"] 0]
	set current_graph_data_length [string length "$current_line_graph"]
#	puts -nonewline $current_gene_ID_A\t$current_graph_data_length\n
	while {$current_graph_data_length >= 1} {
	set current_graph_data [lindex [split $current_line_graph "|"] $current_graph_data_length]

	set current_gene_ID_B [lindex [split $current_graph_data ":"] 0]
	set current_gene_similarity [lindex [split $current_graph_data ":"] 1]

	set fill_color white

	if {$current_gene_similarity >= $level_of_similarity_1} {
	    set fill_color $color_of_similarity_1
	    set line_thickness 3
	}
	if {$current_gene_similarity >= $level_of_similarity_2 && $current_gene_similarity < $level_of_similarity_1} {
	    set fill_color $color_of_similarity_2
	    set line_thickness 2
	}
	if {$current_gene_similarity >= $level_of_similarity_3 && $current_gene_similarity < $level_of_similarity_2} {
	    set fill_color $color_of_similarity_3
	    set line_thickness 1
	}
	if {$current_gene_similarity >= $level_of_similarity_4 && $current_gene_similarity < $level_of_similarity_3} {
	    set fill_color $color_of_similarity_4
	    set line_thickness 1
	}
	if {$current_gene_similarity >= $level_of_similarity_5 && $current_gene_similarity < $level_of_similarity_4} {
	    set fill_color $color_of_similarity_5
	    set line_thickness 1
	}
	if {$current_gene_similarity >= $level_of_similarity_6 && $current_gene_similarity < $level_of_similarity_5} {
	    set fill_color $color_of_similarity_6
	    set line_thickness 1
	}
	if {$current_gene_similarity >= $level_of_similarity_6} {

	set firstNode [.phylo.canvas find withtag $current_gene_ID_A]
	set curNode [.phylo.canvas find withtag $current_gene_ID_B]
	if {($firstNode != "") && ($curNode != "")} {
	    set edge_tag "$edgeCounter"
	    mkEdge $firstNode $curNode $fill_color $line_thickness \
		$current_gene_ID_A $current_gene_ID_B \
		$edge_tag $current_gene_similarity
	    .phylo.canvas itemconfigure $curNode -fill blue -outline cyan
	    .phylo.canvas itemconfigure $firstNode -fill blue -outline cyan
	    unset current_gene_ID_B
	    }
	}
	    incr current_graph_data_length -1
	}
    }
    bell
    close $graph_data_file_open
##### COMMENT ON WINDOWS CURSOR CONFIG #####
    .phylo.canvas config -cursor {tcross white}
}

proc showWeight {current_edge_weight layer3} {

    $layer3 delete 0 end
    $layer3 insert end $current_edge_weight

}

proc showName {current_gene_ID layer1} {

##### COMMENT ON WINDOWS CURSOR CONFIG #####
    .phylo.canvas config -cursor {plus white}
    $layer1 delete 0 end
    $layer1 insert end $current_gene_ID
}

proc highLightGene { w tn } {

    set old_node_colour [lindex [$w itemconfig $tn -fill] 4]
    set old_node_fill [lindex [$w itemconfig $tn -outline] 4]
    $w itemconfigure $tn -fill "purple" -outline "yellow"
}

proc highLightGene_Back { w tn old_node_colour old_node_fill } {
    $w itemconfigure $tn -fill $old_node_colour -outline $old_node_fill
}

proc HighLightEdge { w current current_gene_similarity} {

##### COMMENT ON WINDOWS CURSOR CONFIG #####
    $w config -cursor {cross cyan}

    set old_colour [lindex [$w itemconfig $current -fill] 4]
    $w itemconfigure $current -fill "red"
#    $w itemconfigure $current -fill $old_colour
    Global_XY_Coords $current

    showWeight $current_gene_similarity .geneid.weight_entry

}

proc Global_XY_Coords {current} {

    global sim_x sim_y sim_curX sim_curY
    
    .phylo.canvas bind $current <Motion> {
	set sim_x [.phylo.canvas canvasx %x]
	set sim_x [expr int(round($sim_x))]
	set sim_y [.phylo.canvas canvasy %y]
	set sim_y [expr int(round($sim_y))]
	set sim_curX [.phylo.canvas canvasx %x]
	set sim_curX [expr int(round($sim_curX))]
	set sim_curY [.phylo.canvas canvasy %y]
	set sim_curY [expr int(round($sim_curY))]
    }
}

proc HighLightEdge_Back { w current fill_color} {

##### COMMENT ON WINDOWS CURSOR CONFIG #####
    $w config -cursor {tcross white}

    clearWeight .geneid.weight_entry
    $w itemconfigure $current -fill $fill_color
}

proc clearName { layer1 layer2 } {

##### COMMENT ON WINDOWS CURSOR CONFIG #####
    .phylo.canvas config -cursor {tcross white}
    $layer1 delete 0 end
    $layer2 delete 0 end
}

proc clearWeight { layer3 } {

    $layer3 delete 0 end

}

proc showDegree {edge_counter layer2} {

##### COMMENT ON WINDOWS CURSOR CONFIG #####
    .phylo.canvas config -cursor {plus white}
    $layer2 delete 0 end
    $layer2 insert end $edge_counter
}

proc Dots_Coord_Generator  {} {

    global graph_data_file dots_coord_file
    set graph_data_file_open [open $graph_data_file "r"]
    set nodes_coord_open [open $dots_coord_file "w"]

    while {[gets $graph_data_file_open current_line_graph] >= 0 } {
	set current_gene_ID [lindex [split $current_line_graph "|"] 0]
	set the_Node [.phylo.canvas find withtag $current_gene_ID]
	if {$the_Node == ""} {
	    continue
	}
	make_Nodes_Coords $the_Node $nodes_coord_open $current_gene_ID
    }
    close $graph_data_file_open
    close $nodes_coord_open
}

proc make_Nodes_Coords {the_Node nodes_coord_open current_gene_ID} {

    global nodeX nodeY
    puts -nonewline $nodes_coord_open $current_gene_ID\t$nodeX($the_Node)\t$nodeY($the_Node)\n

}

proc Nodes_Coord_Dialog {} {

    destroy .node_xy
    toplevel .node_xy

    wm title .node_xy "Node Coordinates" 
    wm protocol .node_xy WM_DELETE_WINDOW "destroy .node_xy"
    wm minsize .node_xy 360 150

    global dots_coord_file dots_coord_file_restore
#####
    label .node_xy.f_to_save_label -text "Save Coords to:" -width 14 -bg gray
    place .node_xy.f_to_save_label -x 20 -y 20

    entry .node_xy.f_to_save_entry  -relief sunken \
	    -textvariable dots_coord_file -bg lightyellow -width 17
    place .node_xy.f_to_save_entry -x 130 -y 20

    button .node_xy.save -text "Save" -bg yellow3 \
	    -width 6 -command {Dots_Coord_Generator}
    place .node_xy.save -x 270 -y 18
#####
    label .node_xy.f_restore_label -text "Restore Coords:" -width 14 -bg gray
    place .node_xy.f_restore_label -x 20 -y 60

    entry .node_xy.f_restore_entry  -relief sunken \
	    -textvariable dots_coord_file_restore -bg lightyellow -width 17
    place .node_xy.f_restore_entry -x 130 -y 60

    button .node_xy.restore -text "Restore" -bg yellow3 \
	    -width 6 -command {Dots_Coord_Restore}
    place .node_xy.restore -x 270 -y 58
#####
    button .node_xy.howto -text "HowTo" -bg gray -command {How_To_Save}
    place .node_xy.howto -x 40 -y 120

    button .node_xy.dismiss -text "Dismiss" -bg gray -command {destroy .node_xy}
    place .node_xy.dismiss -x 200 -y 120
}

proc How_To_Save {} {

    destroy .how_to_save
    toplevel .how_to_save
    wm title .how_to_save "HowTo Save and Restore" 
    wm protocol .how_to_save WM_DELETE_WINDOW "destroy .how_to_save"

    label .how_to_save.text -text "
You may save coordinates of nodes for current 
project and restore those coordinates when 
you re-run new project with the same node's 
ID set.
WARNING! Only coordinates and node IDs are 
saved. Labels or other graphics added 
on canvas by hand are not saved in 
\"phylo_coords.txt\" file.
"
    button .how_to_save.dismiss -text "Dismiss" \
	-command "destroy .how_to_save" -bg gray -width 8

    pack .how_to_save.text -padx 7 -pady 7
    pack .how_to_save.dismiss -padx 3 -pady 3
}

proc Dots_Coord_Restore {} {

    global nodeX nodeY
    global dots_coord_file_restore

    set f_open_restore [open $dots_coord_file_restore "r"]

    while {[gets $f_open_restore current_line_coord] >= 0 } {
	set current_gene_ID [lindex [split $current_line_coord] 0]
	set coordinates_x [lindex [split $current_line_coord] 1]
	set coordinates_y [lindex [split $current_line_coord] 2]
	set the_Node [.phylo.canvas find withtag $current_gene_ID]
	if {$the_Node == ""} {
	    continue
	}
	set move_me_X [expr $coordinates_x - $nodeX($the_Node)]
	set move_me_Y [expr $coordinates_y - $nodeY($the_Node)]
	if {$move_me_X == 0 && $move_me_Y == 0} {
	    continue
	}
	if {$move_me_X > 0} {
	    while {$move_me_X > 0} {
	    moveNode $the_Node 1 0
	    incr move_me_X -1
	    update
	    }
	}
	if {$move_me_X < 0} {
	    while {$move_me_X < 0} {
	    moveNode $the_Node -1 0
	    incr move_me_X 1
	    update
	    }
	}
	if {$move_me_Y > 0} {
	    while {$move_me_Y > 0} {
	    moveNode $the_Node 0 1
	    incr move_me_Y -1
	    update
	    }
	}
	if {$move_me_Y < 0} {
	    while {$move_me_Y < 0} {
	    moveNode $the_Node 0 -1
	    incr move_me_Y 1
	    update
	    }
	}
#	moveNode $the_Node $move_me_X $move_me_Y
#	update
#	after 1
    }
    close $f_open_restore
    bell
}

proc HTML_Image_Map_Generator {} {

    global graph_data_file html_image_map
    set graph_data_file_open [open $graph_data_file "r"]
    set html_image_map_open [open $html_image_map "w"]

    while {[gets $graph_data_file_open current_line_graph] >= 0 } {
	set current_gene_ID [lindex [split $current_line_graph "|"] 0]
	set the_Node [.phylo.canvas find withtag $current_gene_ID]
	if {$the_Node == ""} {
	    continue
	}
	make_HTML $the_Node $html_image_map_open $current_gene_ID
    }
    close $graph_data_file_open
    close $html_image_map_open
}

proc make_HTML {the_Node html_image_map_open current_gene_ID} {
    global nodeX nodeY
    puts -nonewline $html_image_map_open "<AREA SHAPE=\"RECT\" COORDS=\""
    puts -nonewline $html_image_map_open "[expr $nodeX($the_Node)-5],[expr $nodeY($the_Node)-5],"
    puts -nonewline $html_image_map_open "[expr $nodeX($the_Node)+5],[expr $nodeY($the_Node)+5]\" "
    puts -nonewline $html_image_map_open "HREF=\"http\:\/\/my_url\/"
    puts -nonewline $html_image_map_open "$current_gene_ID"
    puts -nonewline $html_image_map_open "\">\n"
}

proc Print_Gene_ID {current_gene_ID curX curY x y} {

    set label_tag "label"

    global nodeX nodeY edgeFirst edgeSecond

    set new [.phylo.canvas create text [expr $curX+3] [expr $curY-3] -text $current_gene_ID \
	    -font "helvetica" -anchor sw -fill cyan -tag $current_gene_ID\_$label_tag]

    set nodeX($new) $x
    set nodeY($new) $y
    set edgeFirst($new) {}
    set edgeSecond($new) {}

    .phylo.canvas bind $current_gene_ID\_$label_tag <Button-1> {
    set curX %x
    set curY %y
    }
    .phylo.canvas bind $current_gene_ID\_$label_tag <B1-Motion> {
	moveNode [.phylo.canvas find withtag current] [expr %x-$curX] [expr %y-$curY]
	set curX %x
	set curY %y
    }
}

proc Print_Edge_Weight {edge_tag} {

    global sim_x sim_y sim_curX sim_curY

    set x $sim_x
    set y $sim_y
    set curX $sim_curX
    set curY $sim_curY

    set label_tag "label"

    global nodeX nodeY edgeFirst edgeSecond

    set new [.phylo.canvas create text [expr $curX+3] [expr $curY-3] -text "$edge_tag" \
	    -font "helvetica 12 bold" -anchor sw -fill violet -tag $edge_tag\_$label_tag]

    set nodeX($new) $x
    set nodeY($new) $y
    set edgeFirst($new) {}
    set edgeSecond($new) {}

    .phylo.canvas bind $edge_tag\_$label_tag <Button-1> {
    set curX %x
    set curY %y
    }
    .phylo.canvas bind $edge_tag\_$label_tag <B1-Motion> {
	moveNode [.phylo.canvas find withtag current] [expr %x-$curX] [expr %y-$curY]
	set curX %x
	set curY %y
    }
}

proc Gene_ID_Window {} {

    toplevel .geneid
    .geneid config -bg black
    wm transient .geneid .phylo.canvas
    wm title .geneid "Graph Info"
    wm protocol .geneid WM_DELETE_WINDOW "destroy .geneid"

    label .geneid.label -text "Node ID: " -bg black -fg lightblue
    entry .geneid.entry -relief sunken -textvariable current_gene_ID \
	    -bg blue4 -fg yellow -insertbackground lightyellow
    grid .geneid.label .geneid.entry
    
    label .geneid.degree_label -text "Degree: " -bg black -fg lightblue
    entry .geneid.degree_entry -relief sunken -textvariable current_gene_degree \
	    -bg blue4 -fg yellow -insertbackground lightyellow
    grid .geneid.degree_label .geneid.degree_entry

    label .geneid.empty -text "" -bg black -fg lightblue
    grid .geneid.empty

    label .geneid.weight_label -text "Edge Weight: " -bg black -fg lightblue
    entry .geneid.weight_entry -relief sunken -textvariable current_edge_weight \
	    -bg blue4 -fg yellow -insertbackground lightyellow
    grid .geneid.weight_label .geneid.weight_entry
}

proc Shake_the_Node {current_gene_ID} {
    set k 20
    set l 20
    set m 20
    set n 20
    set o 20
    set p 20

    set old_node_colour [lindex [.phylo.canvas itemconfig $current_gene_ID -fill] 4]
    set old_node_fill [lindex [.phylo.canvas itemconfig $current_gene_ID -outline] 4]
    .phylo.canvas itemconfigure $current_gene_ID -fill "red" -outline "yellow"

    while {$k >= 0} {
	moveNode_Slow $current_gene_ID 0 -1
	update
	incr k -1
    }
    while {$l >= 0} {
	moveNode_Slow $current_gene_ID 1 1
	update
	incr l -1
    }
    while {$m >= 0} {
	moveNode_Slow $current_gene_ID -1 1
	update
	incr m -1
    }
    while {$n >= 0} {
	moveNode_Slow $current_gene_ID -1 -1
	update
	incr n -1
    }
    while {$o >= 0} {
	moveNode_Slow $current_gene_ID 1 -1
	update
	incr o -1
    }
    while {$p >= 0} {
	moveNode_Slow $current_gene_ID 0 1
	update
	incr p -1
    }
    bell
    .phylo.canvas itemconfigure $current_gene_ID -fill $old_node_colour -outline $old_node_fill
    clearName .geneid.entry .geneid.degree_entry
}

proc Edge_Counter {} {

    global edges_data_file

    destroy .edge_counter_win
    toplevel .edge_counter_win
    wm title .edge_counter_win "Edge Counter"
    wm protocol .edge_counter_win WM_DELETE_WINDOW "destroy .edge_counter_win"
    wm minsize .edge_counter_win 300 200
#    . config -bg black

    label .edge_counter_win.input_file_label -text "output file:" -bg gray -width 12
    place .edge_counter_win.input_file_label -x 20 -y 20

    entry .edge_counter_win.input_file_entry -relief sunken \
	    -textvariable edges_data_file -bg lightyellow -width 20
    place .edge_counter_win.input_file_entry -x 115 -y 20

    button .edge_counter_win.howto -text "HowTo" -bg gray -width 8 -command {HowTo_Count_Edges}
    place .edge_counter_win.howto -x 60 -y 150

    button .edge_counter_win.dismiss -text "Dismiss" -bg gray -width 8 -command {destroy .edge_counter_win}
    place .edge_counter_win.dismiss -x 160 -y 150

    button .edge_counter_win.doit -text "Count Edges" -bg orange -width 12 -command {Edge_Degree_Counter}
    place .edge_counter_win.doit -x 140 -y 50

}

proc HowTo_Count_Edges {} {

    destroy .how_to_edges
    toplevel .how_to_edges
    wm title .how_to_edges "HowTo Count Edges" 
    wm protocol .how_to_edges WM_DELETE_WINDOW "destroy .how_to_edges"

    label .how_to_edges.text -text "
Each vertex may have some number of edges.
Number of edges for given vertex is a 
vertex degree. To generate file with the 
list of vertex degree for all elements in 
your graph file just click \"Count Edges\".
Program will examine the current canvas 
and generate a new file you specify.
"
    button .how_to_edges.dismiss -text "Dismiss" \
	-command "destroy .how_to_edges" -bg gray -width 8

    pack .how_to_edges.text -padx 7 -pady 7
    pack .how_to_edges.dismiss -padx 3 -pady 3

}

proc Edge_Degree_Counter {} {

    global graph_data_file edges_data_file
    set graph_data_file_open [open $graph_data_file "r"]
    set edges_data_file_open [open $edges_data_file "w"]

    while {[gets $graph_data_file_open current_line_graph] >= 0 } {
	set current_gene_ID [lindex [split $current_line_graph "|"] 0]
	set the_Node [.phylo.canvas find withtag $current_gene_ID]
	if {$the_Node == ""} {
	    continue
	}
#	Edge_Degree_Counter_Movie $the_Node
	Edge_Degree_Counter_Movie $the_Node $edges_data_file_open $current_gene_ID
    }
    close $graph_data_file_open
    close $edges_data_file_open
    bell
    after 100
    bell
}

proc Edge_Degree_Counter_Movie {the_Node edges_data_file_open current_gene_ID} {
    set k 0
    set l 0
    set m 0
    set n 0
    set o 0
    set p 0

    set old_node_colour [lindex [.phylo.canvas itemconfig $current_gene_ID -fill] 4]
    set old_node_fill [lindex [.phylo.canvas itemconfig $current_gene_ID -outline] 4]
    .phylo.canvas itemconfigure $current_gene_ID -fill "red" -outline "yellow"

    while {$k >= 0} {
	moveNode_Slow $the_Node 0 -1 
	update
	incr k -1
    }
    while {$l >= 0} {
	moveNode_Slow $the_Node 1 1 
	update
	incr l -1
    }
    while {$m >= 0} {
	moveNode_Slow $the_Node -1 1 
	update
	incr m -1
    }
    while {$n >= 0} {
	moveNode_Slow $the_Node -1 -1 
	update
	incr n -1
    }
    while {$o >= 0} {
	moveNode_Slow $the_Node 1 -1 
	update
	incr o -1
    }
    while {$p >= 0} {
	moveNode_Slow_Edges $the_Node 0 1 $edges_data_file_open $current_gene_ID
	update
	incr p -1
    }
#    bell
    .phylo.canvas itemconfigure $current_gene_ID -fill $old_node_colour -outline $old_node_fill
    clearName .geneid.entry .geneid.degree_entry
}

proc moveNode_Slow_Edges {node xDist yDist edges_data_file_open current_gene_ID} {
    global nodeX nodeY edgeFirst edgeSecond
    .phylo.canvas move $node $xDist $yDist
    incr nodeX($node) $xDist
    incr nodeY($node) $yDist
    set edge_counter 0
    foreach edge $edgeFirst($node) {
	.phylo.canvas coords $edge $nodeX($node) $nodeY($node) \
	[lindex [.phylo.canvas coords $edge] 2] \
	[lindex [.phylo.canvas coords $edge] 3]
	incr edge_counter
    }
    foreach edge $edgeSecond($node) {
	.phylo.canvas coords $edge [lindex [.phylo.canvas coords $edge] 0] \
	[lindex [.phylo.canvas coords $edge] 1] \
	$nodeX($node) $nodeY($node)
	incr edge_counter
    }
    showDegree $edge_counter .geneid.degree_entry
    puts $edges_data_file_open $current_gene_ID\t$edge_counter
}

proc Events_Binding {} {

    global custom_node_counter genes_range
    set custom_node_counter 1
    bind .phylo.canvas <Control-n> {
	set new_node "Custom_Node_$custom_node_counter"
	mkNode [expr int(round([.phylo.canvas canvasx %x]))] \
		[expr int(round([.phylo.canvas canvasy %y]))] $new_node
	incr custom_node_counter
	incr genes_range
    }

    bind .phylo.canvas <Control-d> {
	set curX %x
	set curY %y
	.phylo.canvas find withtag current
	.phylo.canvas delete current
	clearName .geneid.entry .geneid.degree_entry
    }
}

proc PhyloGrapher_Canvas {} {

    destroy .phylo .geneid

    global horizontal_size vertical_size
    global canvas_background font_color_canvas
    global genes_range graph_radius
    global edgeCounter

    set edgeCounter 0

    set vertical_size $horizontal_size
    set graph_radius [expr $horizontal_size/2 - 45]

    set scrollmax_x [expr $horizontal_size + 20]
    set scrollmax_y [expr $vertical_size + 20]

    Scrolled_Canvas .phylo -width 700 -height 700 \
	-scrollregion "0 0 $scrollmax_x $scrollmax_y" -bg white ; # -bg $canvas_background

    Dummy_Color_Movie

    Gene_ID_Window
    Html_ScreenShot $horizontal_size $vertical_size
    Plot_Genes_on_Canvas $genes_range $graph_radius

    Events_Binding
    Create_Graph_from_Data

    focus .phylo.canvas

##### COMMENT ON WINDOWS CURSOR CONFIG #####
    .phylo.canvas config -cursor {tcross white}

    .phylo.canvas create rect 0 0 $scrollmax_x $scrollmax_y \
	-fill $canvas_background -outline $canvas_background -tag dummy_ps_background
    .phylo.canvas lower dummy_ps_background
}

proc Dummy_Color_Movie {} {

    .phylo.canvas create text 240 300 -fill black -text "PhyloGrapher" \
	    -font "helvetica 32 bold" -anchor sw -tag phylo_banner

    set q 100
    while {$q >= 0} {
	.phylo.canvas configure -bg gray$q
	after 2
	incr q -1
	update
    }
    .phylo.canvas delete phylo_banner
}

proc Html_ScreenShot { horizontal_size vertical_size } {
    .phylo.canvas create line 0 0 [expr $horizontal_size -1] 0 -fill gray50 
    .phylo.canvas create line 0 0 0 [expr $vertical_size -1] -fill gray50
    .phylo.canvas create line   [expr $horizontal_size -1] 0 [expr $horizontal_size -1]\
	                    [expr $vertical_size -1] -fill gray50
    .phylo.canvas create line 0 [expr $vertical_size -1] [expr $horizontal_size -1]\
	                    [expr $vertical_size -1] -fill gray50
}

proc Create_Graph_A {current_seq_ID graph_file_to_write} {

    global matrix_data_file level_of_similarity_6

    set matrix_file_for_graph $matrix_data_file
    set matrix_file_for_graph_open [open $matrix_file_for_graph "r"]
    set identity_graph_limit $level_of_similarity_6
    while {[gets $matrix_file_for_graph_open current_line_graph] >= 0}  {
    set graph_gene_ID_A [lindex $current_line_graph 0]
    set graph_gene_ID_B [lindex $current_line_graph 1]
    set graph_identity [lindex $current_line_graph 2]
    if {$current_seq_ID == $graph_gene_ID_A && $graph_identity >= $identity_graph_limit} {
	puts -nonewline "$graph_gene_ID_B\:$graph_identity\|"
	puts -nonewline $graph_file_to_write "$graph_gene_ID_B\:$graph_identity\|"
        }
    }
    close $matrix_file_for_graph_open
}

proc Generate_Graph_From_Matrix {} {

##### COMMENT ON WINDOWS CURSOR CONFIG #####
    . config -cursor {watch white black}

global genes_id_data_file matrix_data_file graph_data_file
global genes_range

set input_file_to_create_graph $genes_id_data_file
set graph_file $graph_data_file
set genes_range 0

    set input_file_to_open_graph [open "$input_file_to_create_graph" "r"]
    set graph_file_to_write [open "$graph_file" "w"]

    while {[gets $input_file_to_open_graph current_line] >= 0}  {

        set current_seq_ID [lindex $current_line 0]
	puts -nonewline "$current_seq_ID\|"
	puts -nonewline $graph_file_to_write "$current_seq_ID\|"
	Create_Graph_A $current_seq_ID $graph_file_to_write
	puts -nonewline "\n"
	puts -nonewline $graph_file_to_write "\n"
	incr genes_range
	update
    }
close $input_file_to_open_graph
close $graph_file_to_write
bell
##### COMMENT ON WINDOWS CURSOR CONFIG #####
    . config -cursor {top_left_arrow black white}
}

proc Show_Extras {} {

    destroy .extras_win
    toplevel .extras_win
    wm title .extras_win "Extras"
    wm protocol .extras_win WM_DELETE_WINDOW "destroy .extras_win"
    wm minsize .extras_win 320 200
    .extras_win config -bg black

    button .extras_win.canvas_ed -text "Canvas\nEditor" -command {Canvas_Editor} \
	-bg blue4 -fg lightblue -width 7 -height 2
    place .extras_win.canvas_ed -x 20 -y 20

    button .extras_win.node_ed -text "Node\nPainter" -command {Node_Painter} \
	-bg blue4 -fg lightblue -width 7 -height 2
    place .extras_win.node_ed -x 120 -y 20

    button .extras_win.node_coord -text "Node\nCoords" -command {Nodes_Coord_Dialog} \
	-bg blue4 -fg lightblue -width 7 -height 2
    place .extras_win.node_coord -x 220 -y 20

    button .extras_win.edges -text "Edge\nCounter" -command {Edge_Counter} \
	-bg blue4 -fg lightblue -width 7 -height 2
    place .extras_win.edges -x 20 -y 70

    button .extras_win.dismiss -text "Dismiss" -command {destroy .extras_win} \
	-bg gray30 -fg gray75 -width 7 -height 2
    place .extras_win.dismiss -x 120 -y 130

}

proc Node_Painter {} {

    destroy .node_painter
    toplevel .node_painter
    wm title .node_painter "Node Painter"
    wm protocol .node_painter WM_DELETE_WINDOW "destroy .node_painter"
    wm minsize .node_painter 400 230
#    .node_painter config -bg black

    global nodes_group1 nodes_group2 nodes_group3
    global nodes_group4 nodes_group5 nodes_group6

    set nodes_group1 "./Group/my_group1.txt"
    set nodes_group2 "./Group/my_group2.txt"
    set nodes_group3 "./Group/my_group3.txt"
    set nodes_group4 "./Group/my_group4.txt"
    set nodes_group5 "./Group/my_group5.txt"
    set nodes_group6 "./Group/my_group6.txt"

    global color_group1 color_group2 color_group3 
    global color_group4 color_group5 color_group6

    set color_group1 white
    set color_group2 orange
    set color_group3 red
    set color_group4 green
    set color_group5 purple
    set color_group6 blue

#####
    label .node_painter.select_group1_label -text "group1:" -bg gray -width 12
    place .node_painter.select_group1_label -x 10 -y 10

    entry .node_painter.select_group1_entry -relief sunken \
	    -textvariable nodes_group1 -bg lightyellow -width 20
    place .node_painter.select_group1_entry -x 105 -y 10

    entry .node_painter.color_group1_entry -relief sunken \
	    -textvariable color_group1 -bg lightyellow -width 8
    place .node_painter.color_group1_entry -x 260 -y 10

    button .node_painter.paint_group1 -text "paint" -bg yellow3 \
	    -command {Change_Node_Color1}
    place .node_painter.paint_group1 -x 330 -y 8
#####
    label .node_painter.select_group2_label -text "group2:" -bg gray -width 12
    place .node_painter.select_group2_label -x 10 -y 40

    entry .node_painter.select_group2_entry -relief sunken \
	    -textvariable nodes_group2 -bg lightyellow -width 20
    place .node_painter.select_group2_entry -x 105 -y 40

    entry .node_painter.color_group2_entry -relief sunken \
	    -textvariable color_group2 -bg lightyellow -width 8
    place .node_painter.color_group2_entry -x 260 -y 40

    button .node_painter.paint_group2 -text "paint" -bg yellow3 \
	    -command {Change_Node_Color2}
    place .node_painter.paint_group2 -x 330 -y 38
#####
    label .node_painter.select_group3_label -text "group3:" -bg gray -width 12
    place .node_painter.select_group3_label -x 10 -y 70

    entry .node_painter.select_group3_entry -relief sunken \
	    -textvariable nodes_group3 -bg lightyellow -width 20
    place .node_painter.select_group3_entry -x 105 -y 70

    entry .node_painter.color_group3_entry -relief sunken \
	    -textvariable color_group3 -bg lightyellow -width 8
    place .node_painter.color_group3_entry -x 260 -y 70

    button .node_painter.paint_group3 -text "paint" -bg yellow3 \
	    -command {Change_Node_Color3}
    place .node_painter.paint_group3 -x 330 -y 68
#####
    label .node_painter.select_group4_label -text "group4:" -bg gray -width 12
    place .node_painter.select_group4_label -x 10 -y 100

    entry .node_painter.select_group4_entry -relief sunken \
	    -textvariable nodes_group4 -bg lightyellow -width 20
    place .node_painter.select_group4_entry -x 105 -y 100

    entry .node_painter.color_group4_entry -relief sunken \
	    -textvariable color_group4 -bg lightyellow -width 8
    place .node_painter.color_group4_entry -x 260 -y 100

    button .node_painter.paint_group4 -text "paint" -bg yellow3 \
	    -command {Change_Node_Color4}
    place .node_painter.paint_group4 -x 330 -y 98
#####
    label .node_painter.select_group5_label -text "group5:" -bg gray -width 12
    place .node_painter.select_group5_label -x 10 -y 130

    entry .node_painter.select_group5_entry -relief sunken \
	    -textvariable nodes_group5 -bg lightyellow -width 20
    place .node_painter.select_group5_entry -x 105 -y 130

    entry .node_painter.color_group5_entry -relief sunken \
	    -textvariable color_group5 -bg lightyellow -width 8
    place .node_painter.color_group5_entry -x 260 -y 130

    button .node_painter.paint_group5 -text "paint" -bg yellow3 \
	    -command {Change_Node_Color5}
    place .node_painter.paint_group5 -x 330 -y 128
#####
    label .node_painter.select_group6_label -text "group6:" -bg gray -width 12
    place .node_painter.select_group6_label -x 10 -y 160

    entry .node_painter.select_group6_entry -relief sunken \
	    -textvariable nodes_group6 -bg lightyellow -width 20
    place .node_painter.select_group6_entry -x 105 -y 160

    entry .node_painter.color_group6_entry -relief sunken \
	    -textvariable color_group6 -bg lightyellow -width 8
    place .node_painter.color_group6_entry -x 260 -y 160

    button .node_painter.paint_group6 -text "paint" -bg yellow3 \
	    -command {Change_Node_Color6}
    place .node_painter.paint_group6 -x 330 -y 158
#####

    button .node_painter.howto -text "HowTo" -bg gray -width 8 -command {HowTo_Paint_Nodes}
    place .node_painter.howto -x 20 -y 195

    button .node_painter.dismiss -text "Dismiss" -bg gray -width 8 -command {destroy .node_painter}
    place .node_painter.dismiss -x 140 -y 195

}

proc HowTo_Paint_Nodes {} {

    destroy .how_to_paint
    toplevel .how_to_paint
    wm title .how_to_paint "HowTo Paint Nodes" 
    wm protocol .how_to_paint WM_DELETE_WINDOW "destroy .how_to_paint"

    label .how_to_paint.text -text "
To paint nodes in different colors you need
to set up lists of nodes belonging to distinct 
groups and place them in \"Group\" directory. 
By clicking \"paint\" program changes color 
of nodes in a given list.
"
    button .how_to_paint.dismiss -text "Dismiss" \
	-command "destroy .how_to_paint" -bg gray -width 8

    pack .how_to_paint.text -padx 7 -pady 7
    pack .how_to_paint.dismiss -padx 3 -pady 3

}

proc Change_Node_Color1 {} {
    global nodes_group1 color_group1
    set open_current_group [open $nodes_group1 "r"]
    while {[gets $open_current_group current_line_group] >= 0} {
	.phylo.canvas itemconfigure $current_line_group -fill $color_group1
    }
    close $open_current_group
    bell
}
proc Change_Node_Color2 {} {
    global nodes_group2 color_group2
    set open_current_group [open $nodes_group2 "r"]
    while {[gets $open_current_group current_line_group] >= 0} {
	.phylo.canvas itemconfigure $current_line_group -fill $color_group2
    }
    close $open_current_group
    bell
}
proc Change_Node_Color3 {} {
    global nodes_group3 color_group3
    set open_current_group [open $nodes_group3 "r"]
    while {[gets $open_current_group current_line_group] >= 0} {
	.phylo.canvas itemconfigure $current_line_group -fill $color_group3
    }
    close $open_current_group
    bell
}
proc Change_Node_Color4 {} {
    global nodes_group4 color_group4
    set open_current_group [open $nodes_group4 "r"]
    while {[gets $open_current_group current_line_group] >= 0} {
	.phylo.canvas itemconfigure $current_line_group -fill $color_group4
    }
    close $open_current_group
    bell
}
proc Change_Node_Color5 {} {
    global nodes_group5 color_group5
    set open_current_group [open $nodes_group5 "r"]
    while {[gets $open_current_group current_line_group] >= 0} {
	.phylo.canvas itemconfigure $current_line_group -fill $color_group5
    }
    close $open_current_group
    bell
}
proc Change_Node_Color6 {} {
    global nodes_group6 color_group6
    set open_current_group [open $nodes_group6 "r"]
    while {[gets $open_current_group current_line_group] >= 0} {
	.phylo.canvas itemconfigure $current_line_group -fill $color_group6
    }
    close $open_current_group
    bell
}

proc Canvas_Editor {} {

    destroy .c_editor
    toplevel .c_editor

    wm title .c_editor "Canvas Editor" 
    wm protocol .c_editor WM_DELETE_WINDOW "destroy .c_editor"
    wm minsize .c_editor 512 260

global selected_canvas \
	font_color_canvas \
	font_color_label \
	canvas_editor_font_type \
	canvas_editor_font_size \
	canvas_editor_font_style \
	text_tag \
	canvas_editor_text \
	curX curY 

set font_color_label $font_color_canvas
set selected_canvas ".phylo.canvas"
set canvas_editor_text "my lovely node"
set canvas_editor_font_type "helvetica"
set canvas_editor_font_size 12
set canvas_editor_font_style normal
set text_tag text_tag_01
set curX 125
set curY 125

global	graph_style \
	graph_tag \
	graphX1 graphX2 \
	graphY1 graphY2 \
	graph_fill_color \
	graph_outline_color

set graph_style "oval"
set graph_tag "graph_tag_01"
set graphX1 100 
set graphY1 100
set graphX2 24
set graphY2 24
set graph_fill_color ""
set graph_outline_color "white"

###########################################################################

    label .c_editor.select_text_label -text "select text:" -bg yellow3 -width 12
    place .c_editor.select_text_label -x 10 -y 10

    entry .c_editor.select_text_entry -relief sunken \
	    -textvariable canvas_editor_text -bg gold -width 22
    place .c_editor.select_text_entry -x 105 -y 10

    label .c_editor.select_canvas -text "select canvas:" -bg gray -width 14
    place .c_editor.select_canvas -x 280 -y 10

    entry .c_editor.select_canvas_entry -relief sunken \
	    -textvariable selected_canvas -bg lightblue -width 15
    place .c_editor.select_canvas_entry -x 390 -y 10

########################

    label .c_editor.text_color_label -text "text color:" -bg gray -width 12
    place .c_editor.text_color_label -x 10 -y 40

    entry .c_editor.text_color_entry -relief sunken \
	    -textvariable font_color_label -bg lightyellow -width 20
    place .c_editor.text_color_entry -x 105 -y 40

    label .c_editor.select_graph_label -text "Graphical Label:" -bg yellow3 -width 18
    place .c_editor.select_graph_label -x 330 -y 40

########################

    label .c_editor.text_type_label -text "font type:" -bg gray -width 12
    place .c_editor.text_type_label -x 10 -y 70

    entry .c_editor.text_type_entry -relief sunken \
	    -textvariable canvas_editor_font_type -bg lightyellow -width 20
    place .c_editor.text_type_entry -x 105 -y 70

    label .c_editor.graph_style_label -text "shape:" -bg gray -width 12
    place .c_editor.graph_style_label -x 300 -y 70

    entry .c_editor.graph_style_entry -relief sunken \
	    -textvariable graph_style -bg lightyellow -width 14
    place .c_editor.graph_style_entry -x 395 -y 70

########################

    label .c_editor.text_size_label -text "font size:" -bg gray -width 12
    place .c_editor.text_size_label -x 10 -y 100

    entry .c_editor.text_size_entry -relief sunken \
	    -textvariable canvas_editor_font_size -bg lightyellow -width 20
    place .c_editor.text_size_entry -x 105 -y 100

    label .c_editor.graph_fill_label -text "fill:" -bg gray -width 12
    place .c_editor.graph_fill_label -x 300 -y 100

    entry .c_editor.graph_fill_entry -relief sunken \
	    -textvariable graph_fill_color -bg lightyellow -width 14
    place .c_editor.graph_fill_entry -x 395 -y 100

########################

    label .c_editor.text_style_label -text "font style:" -bg gray -width 12
    place .c_editor.text_style_label -x 10 -y 130

    entry .c_editor.text_style_entry -relief sunken \
	    -textvariable canvas_editor_font_style -bg lightyellow -width 20
    place .c_editor.text_style_entry -x 105 -y 130

    label .c_editor.graph_outline_label -text "outline:" -bg gray -width 12
    place .c_editor.graph_outline_label -x 300 -y 130

    entry .c_editor.graph_outline_entry -relief sunken \
	    -textvariable graph_outline_color -bg lightyellow -width 14
    place .c_editor.graph_outline_entry -x 395 -y 130

########################

    label .c_editor.current_X_label -text "current X:" -bg gray -width 12
    place .c_editor.current_X_label -x 10 -y 160

    label .c_editor.current_Y_label -text "current Y:" -bg gray -width 12
    place .c_editor.current_Y_label -x 10 -y 190

    entry .c_editor.current_X_entry -relief sunken \
	    -textvariable curX -bg lightyellow -width 20
    place .c_editor.current_X_entry -x 105 -y 160

    entry .c_editor.current_Y_entry -relief sunken \
	    -textvariable curY -bg lightyellow -width 20
    place .c_editor.current_Y_entry -x 105 -y 190

    label .c_editor.graphX1 -text "X1" -bg gray -width 4
    place .c_editor.graphX1 -x 300 -y 160

    label .c_editor.graphY1 -text "Y1" -bg gray -width 4
    place .c_editor.graphY1 -x 300 -y 190

    label .c_editor.graphX2 -text "X2" -bg gray -width 4
    place .c_editor.graphX2 -x 400 -y 160

    label .c_editor.graphY2 -text "Y2" -bg gray -width 4
    place .c_editor.graphY2 -x 400 -y 190

    entry .c_editor.graphX1_entry -relief sunken \
	    -textvariable graphX1 -bg lightyellow -width 6
    place .c_editor.graphX1_entry -x 340 -y 160

    entry .c_editor.graphY1_entry -relief sunken \
	    -textvariable graphY1 -bg lightyellow -width 6
    place .c_editor.graphY1_entry -x 340 -y 190

    entry .c_editor.graphX2_entry -relief sunken \
	    -textvariable graphX2 -bg lightyellow -width 6
    place .c_editor.graphX2_entry -x 440 -y 160

    entry .c_editor.graphY2_entry -relief sunken \
	    -textvariable graphY2 -bg lightyellow -width 6
    place .c_editor.graphY2_entry -x 440 -y 190

########################

#    label .c_editor.tag_ID_label -text "active tag ID:" -bg yellow3 -width 12
#    place .c_editor.tag_ID_label -x 10 -y 220

#    entry .c_editor.tag_ID_entry -relief sunken \
#	    -textvariable text_tag -bg gold -width 20
#    place .c_editor.tag_ID_entry -x 105 -y 220

#    label .c_editor.graph_ID_label -text "active tag ID:" -bg yellow3 -width 12
#    place .c_editor.graph_ID_label -x 300 -y 220

#    entry .c_editor.graph_ID_entry -relief sunken \
#	    -textvariable graph_tag -bg gold -width 14
#    place .c_editor.graph_ID_entry -x 395 -y 220

########################

    button .c_editor.submit_text -text "Add Text to Canvas" \
	    -command {Place_Text_on_Canvas} -bg orange -width 17
    place .c_editor.submit_text -x 20 -y 220

    button .c_editor.submit_graphics -text "Add Label to Canvas" \
	    -command {Add_Graphics_to_Canvas} -bg orange -width 17
    place .c_editor.submit_graphics -x 340 -y 220

    button .c_editor.how_to -text "HowTo" \
	    -command {How_To_Edit_Canvas} -bg gray -width 6
    place .c_editor.how_to -x 180 -y 220

    button .c_editor.dismiss -text "Dismiss" \
	    -command {destroy .c_editor} -bg gray -width 6
    place .c_editor.dismiss -x 260 -y 220

###########################################################################

}

proc Move_Text_Label { node xDist yDist } {

    global selected_canvas curX curY

    $selected_canvas move $node $xDist $yDist
    incr curX $xDist
    incr curY $yDist
}

proc Move_Graph_Label { node xDist1 yDist1 } {

    global selected_canvas \
	graphX1 graphX2 \
	graphY1 graphY2

    $selected_canvas move $node $xDist1 $yDist1
    incr graphX1 $xDist1
    incr graphY1 $yDist1
}

proc Place_Text_on_Canvas {} {

global selected_canvas text_tag \
	curX curY canvas_editor_text \
	font_color_canvas font_color_label \
	canvas_editor_font_type \
	canvas_editor_font_size \
	canvas_editor_font_style

$selected_canvas create text $curX $curY -text "$canvas_editor_text" \
-font "$canvas_editor_font_type $canvas_editor_font_size $canvas_editor_font_style" \
-anchor sw -fill $font_color_label -tag $text_tag

$selected_canvas bind $text_tag <Button-1> {
    set curX [expr int(round([.phylo.canvas canvasx %x]))]
    set curY [expr int(round([.phylo.canvas canvasy %y]))]
    }

$selected_canvas bind $text_tag <B1-Motion> {
Move_Text_Label [$selected_canvas find withtag current] \
	[expr [expr int(round([.phylo.canvas canvasx %x]))] - $curX] \
	[expr [expr int(round([.phylo.canvas canvasy %y]))] - $curY]
    set curX [expr int(round([.phylo.canvas canvasx %x]))]
    set curY [expr int(round([.phylo.canvas canvasy %y]))]
    }
}

proc Add_Graphics_to_Canvas {} {

    global	graph_style \
	    graph_tag \
	    graphX1 graphX2 \
	    graphY1 graphY2 \
	    graph_fill_color \
	    graph_outline_color \
	    selected_canvas

    $selected_canvas create $graph_style $graphX1 $graphY1 \
	[expr $graphX1 + $graphX2] \
	[expr $graphY1 + $graphY2] \
	-fill $graph_fill_color -outline $graph_outline_color -tag $graph_tag

    $selected_canvas bind $graph_tag <Button-1> {
    set graphX1 [expr int(round([.phylo.canvas canvasx %x]))]
    set graphY1 [expr int(round([.phylo.canvas canvasy %y]))]
    }

    $selected_canvas bind $graph_tag <B1-Motion> {
	Move_Graph_Label [$selected_canvas find withtag current] \
	[expr [expr int(round([.phylo.canvas canvasx %x]))] - $graphX1] \
	[expr [expr int(round([.phylo.canvas canvasy %y]))] - $graphY1] 
    set graphX1 [expr int(round([.phylo.canvas canvasx %x]))]
    set graphY1 [expr int(round([.phylo.canvas canvasy %y]))]
    }
}

proc How_To_Edit_Canvas {} {

    destroy .how_to_edit_canvas
    toplevel .how_to_edit_canvas
    wm title .how_to_edit_canvas "Canvas Editing HowTo" 
    wm protocol .how_to_edit_canvas WM_DELETE_WINDOW "destroy .how_to_edit_canvas"

    label .how_to_edit_canvas.text -text "
By using this Canvas Editor you may edit and finish 
your project by adding text and graphical labels over 
the graph generated by PhyloGrapher. 
By left-button mouse click you may move the text and 
labels over PhyloGrapher canvas. By pressing \"Cntrl-d\" 
key you may delete the text and labels you don't need.
"
    button .how_to_edit_canvas.dismiss -text "Dismiss" \
	-command "destroy .how_to_edit_canvas" -bg gray -width 8

    pack .how_to_edit_canvas.text -padx 7 -pady 7
    pack .how_to_edit_canvas.dismiss -padx 3 -pady 3
}

#############################################
#             MAIN BODY                     #
#############################################

wm title . "PhyloGrapher"
wm protocol . WM_DELETE_WINDOW "destroy ."
wm minsize . 500 440
. config -bg black

set tcl_precision 6

set phylo_logo [image create photo -file ./Images/PhyloGrapherLogo.gif]
label .phylo_logo -image $phylo_logo -bg black
place .phylo_logo -x 24 -y 12

# VARIABLES

global canvas_background font_color_canvas
global horizontal_size vertical_size
global genes_range graph_radius
global genes_id_data_file matrix_data_file 
global graph_data_file html_image_map edges_data_file
global dots_coord_file dots_coord_file_restore

global level_of_similarity_1 level_of_similarity_2 level_of_similarity_3
global level_of_similarity_4 level_of_similarity_5 level_of_similarity_6

global color_of_similarity_1 color_of_similarity_2 color_of_similarity_3
global color_of_similarity_4 color_of_similarity_5 color_of_similarity_6

global font_color_canvas
global edgeCounter
global fake_identity_counter

set font_color_canvas white
set edgeCounter 0
set fake_identity_counter 1

set level_of_similarity_1 0.90
set level_of_similarity_2 0.80
set level_of_similarity_3 0.70
set level_of_similarity_4 0.675
set level_of_similarity_5 0.65
set level_of_similarity_6 0.625

set color_of_similarity_1 white
set color_of_similarity_2 gold
set color_of_similarity_3 orange
set color_of_similarity_4 gray75
set color_of_similarity_5 gray50
set color_of_similarity_6 gray25

set horizontal_size 700
set canvas_background black

set genes_id_data_file "./Matrix/My_ID_List.txt"
set matrix_data_file "./Matrix/My_Matrix_File.txt"
set graph_data_file "./Graph/My_Graph_File.txt"
set post_script_file "phylo_postscript.ps"
set html_image_map "phylo_html_map.txt"
set dots_coord_file "phylo_coords.txt"
set dots_coord_file_restore "phylo_coords_r.txt"
set edges_data_file "phylo_edges.txt"
set genes_range 0

# BUTTONS

##########################################

button .howto -text "HowTo" -command {HowTo_PhyloWebify} -bg blue4 -fg lightblue -width 7
place .howto -x 310 -y 45

button .about -text "About" -command {About_PhyloGrapher} -bg blue4 -fg lightblue -width 7
place .about -x 310 -y 10

button .exit -text "Exit" -command {destroy .} \
	-bg gray30 -height 1 -fg gray75 -width 7
place .exit -x 405 -y 10

button .extras -text "Extras" -command {Show_Extras} \
	-bg blue4 -height 1 -fg lightblue -width 7
place .extras -x 405 -y 45

##########################################

label .id_label -text "List of Genes:" -bg black -fg gray75 -width 13
place .id_label -x 12 -y 90

label .matrix_label -text "Matrix File:" -bg black -fg gray75 -width 12
place .matrix_label -x 12 -y 120

label .graph_label -text "Graph File:" -bg gray25 -fg gray80 -width 12
place .graph_label -x 12 -y 150

entry .id_entry -relief sunken -textvariable genes_id_data_file \
	-bg blue4 -fg cyan -width 32 -insertbackground lightyellow
place .id_entry -x 120 -y 90

entry .matrix_entry -relief sunken -textvariable matrix_data_file \
	-bg blue4 -fg cyan -width 32 -insertbackground lightyellow
place .matrix_entry -x 120 -y 120

entry .graph_entry -relief sunken -textvariable graph_data_file \
	-bg blue4 -fg white -width 32 -insertbackground lightyellow
place .graph_entry -x 120 -y 150

button .generate_graph -text "Generate Graph\nfrom Matrix" -command {Generate_Graph_From_Matrix} \
		-bg blue4 -fg lightblue -width 12 -height 2
place .generate_graph -x 372 -y 95

label .gene_number_label -text "Number of Nodes:" -bg black -fg gray75 -width 16
place .gene_number_label -x 368 -y 150

entry .gene_number_entry -relief sunken -textvariable genes_range \
	-bg black -fg gold -width 12 -insertbackground lightyellow
place .gene_number_entry -x 378 -y 170

label .edges_number_label -text "Number of Edges:" -bg black -fg gray75 -width 16
place .edges_number_label -x 368 -y 200

label .edges_counter_label -textvariable edgeCounter -bg black -fg gold -width 16
place .edges_counter_label -x 368 -y 220

##########################################

label .identity_color_map -text "Relationship Color Map:" \
	-bg gray25 -fg gray80 -width 30
place .identity_color_map -x 30 -y 190

label .level_1 -text "1" -bg gray25 -fg gray80 -width 2
place .level_1 -x 12 -y 220

label .level_0 -text "1.0 -" -bg gray25 -fg white -width 5
place .level_0 -x 45 -y 220

entry .level_1_entry -relief sunken -textvariable level_of_similarity_1 \
	-bg gray25 -fg white -width 5 -insertbackground lightyellow
place .level_1_entry -x 100 -y 220

entry .color_1 -relief sunken -textvariable color_of_similarity_1 \
	-bg gray25 -fg white -width 10 -insertbackground lightyellow
place .color_1 -x 170 -y 220
###
label .level_2 -text "2" -bg gray25 -fg gray80 -width 2
place .level_2 -x 12 -y 250

entry .level_1a_entry -relief sunken -textvariable level_of_similarity_1 \
	-bg gray25 -fg white -width 5 -insertbackground lightyellow
place .level_1a_entry -x 45 -y 250

entry .level_2_entry -relief sunken -textvariable level_of_similarity_2 \
	-bg gray25 -fg white -width 5 -insertbackground lightyellow
place .level_2_entry -x 100 -y 250

entry .color_2 -relief sunken -textvariable color_of_similarity_2 \
	-bg gray25 -fg white -width 10 -insertbackground lightyellow
place .color_2 -x 170 -y 250
###
label .level_3 -text "3" -bg gray25 -fg gray80 -width 2
place .level_3 -x 12 -y 280

entry .level_2a_entry -relief sunken -textvariable level_of_similarity_2 \
	-bg gray25 -fg white -width 5 -insertbackground lightyellow
place .level_2a_entry -x 45 -y 280

entry .level_3_entry -relief sunken -textvariable level_of_similarity_3 \
	-bg gray25 -fg white -width 5 -insertbackground lightyellow
place .level_3_entry -x 100 -y 280

entry .color_3 -relief sunken -textvariable color_of_similarity_3 \
	-bg gray25 -fg white -width 10 -insertbackground lightyellow
place .color_3 -x 170 -y 280
###
label .level_4 -text "4" -bg gray25 -fg gray80 -width 2
place .level_4 -x 12 -y 310

entry .level_3a_entry -relief sunken -textvariable level_of_similarity_3 \
	-bg gray25 -fg white -width 5 -insertbackground lightyellow
place .level_3a_entry -x 45 -y 310

entry .level_4_entry -relief sunken -textvariable level_of_similarity_4 \
	-bg gray25 -fg white -width 5 -insertbackground lightyellow
place .level_4_entry -x 100 -y 310

entry .color_4 -relief sunken -textvariable color_of_similarity_4 \
	-bg gray25 -fg white -width 10 -insertbackground lightyellow
place .color_4 -x 170 -y 310
###
label .level_5 -text "5" -bg gray25 -fg gray80 -width 2
place .level_5 -x 12 -y 340

entry .level_4a_entry -relief sunken -textvariable level_of_similarity_4 \
	-bg gray25 -fg white -width 5 -insertbackground lightyellow
place .level_4a_entry -x 45 -y 340

entry .level_5_entry -relief sunken -textvariable level_of_similarity_5 \
	-bg gray25 -fg white -width 5 -insertbackground lightyellow
place .level_5_entry -x 100 -y 340

entry .color_5 -relief sunken -textvariable color_of_similarity_5 \
	-bg gray25 -fg white -width 10 -insertbackground lightyellow
place .color_5 -x 170 -y 340
###
label .level_6 -text "6" -bg gray25 -fg gray80 -width 2
place .level_6 -x 12 -y 370

entry .level_5a_entry -relief sunken -textvariable level_of_similarity_5 \
	-bg gray25 -fg white -width 5 -insertbackground lightyellow
place .level_5a_entry -x 45 -y 370

entry .level_6_entry -relief sunken -textvariable level_of_similarity_6 \
	-bg gray25 -fg white -width 5 -insertbackground lightyellow
place .level_6_entry -x 100 -y 370

entry .color_6 -relief sunken -textvariable color_of_similarity_6 \
	-bg gray25 -fg white -width 10 -insertbackground lightyellow
place .color_6 -x 170 -y 370
###
label .level_7 -text "no lines below:" -bg gray25 -fg gray80 -width 15
place .level_7 -x 12 -y 400

entry .level_6a_entry -relief sunken -textvariable level_of_similarity_6 \
	-bg gray25 -fg white -width 5 -insertbackground lightyellow
place .level_6a_entry -x 140 -y 400

##########################################

label .image_size_label -text "vertical and horizontal\nimage size:" -bg gray25 -fg gray80 -width 20
place .image_size_label -x 280 -y 250

entry .image_size_entry -relief sunken -textvariable horizontal_size \
	-bg gray25 -fg white -width 5 -insertbackground lightyellow
place .image_size_entry -x 440 -y 255

##########################################

label .ps_file_label -text "PostScript file:" -bg gray25 -fg gray80 -width 20
place .ps_file_label -x 280 -y 300

entry .ps_file_entry -relief sunken -textvariable post_script_file \
	-bg gray25 -fg white -width 20 -insertbackground lightyellow
place .ps_file_entry -x 280 -y 330

button .ps_button -text "Make\nPS" -bg blue4 -fg lightblue -width 3 -height 2 \
	-command {
    .phylo.canvas create rect 0 0 [expr $horizontal_size + 20] [expr $vertical_size + 20]\
	-fill $canvas_background -outline $canvas_background -tag dummy_ps_background
    .phylo.canvas lower dummy_ps_background
    .phylo.canvas postscript -height $vertical_size -width $horizontal_size \
	-x -1 -y 0 -file $post_script_file
    }
place .ps_button -x 440 -y 305

##########################################

label .html_file_label -text "HTML image map:" -bg gray25 -fg gray80 -width 20
place .html_file_label -x 280 -y 370

entry .html_file_entry -relief sunken -textvariable html_image_map \
	-bg gray25 -fg white -width 20 -insertbackground lightyellow
place .html_file_entry -x 280 -y 400

button .html -text "Make\nHTML" -command {HTML_Image_Map_Generator} \
	-bg blue4 -fg lightblue -width 3 -height 2
place .html -x 440 -y 375

###########################################

button .startup -text "Run" -command {PhyloGrapher_Canvas} \
	-bg blue3 -fg yellow -width 8 -height 2
place .startup -x 270 -y 190

##########################################
